/*  context.h                                                   (CRC:9cb51dee)
 *      relative path: ./I386
 *      creation date: 1992
 *      last modified: Wed Dec 18 07:15:49 1996
 *      header update: Wed Dec 18 07:15:50 1996
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __CONTEXT_H__
#define __CONTEXT_H__


#ifndef ASM
typedef unsigned short segment;

/* per-level stack pointer structure: */
typedef struct {
	vaddr	esp;	/* stack offset */
	segment	ss;	/* stack segment */
	ushort	pad;	
} Stack;

typedef struct {
	/* hardware: */
	ushort	link;	/* link to previous TSS */
	ushort	pad1;	
	Stack	sp[3];	/* stack pointers for each CPL */
	paddr	pdbr;	/* page directory base register (CR3) */
	vaddr	pc;	/* EIP - instruction pointer */
	ulong	eflags;	/* extended CPU flags */
	ulong	reg[8];	/* general registers */
	segment	seg[12];/* segments (only even indices are valid) */	
	segment	ldt;	/* local descriptor table */
	ushort	pad8;
	ushort	debug;	/* bit 0 =  debug trap bit */
	ushort	iomap;	/* base address of I/O map */
	/* software: */
	vaddr	pagedir; /* page directory vbase */
	vaddr	usp;	 /* user stack pointer */
} Context;

#endif

/* offsets for accessing context from assembly: */
#define CONTEXT_PC	32
#define CONTEXT_EFLAGS	36	
#define CONTEXT_SP0	4
#define CONTEXT_SP	56
#define CONTEXT_USP	108

/* general registers for Context: */
#define REG_EAX	0
#define REG_ECX	1
#define REG_EDX	2
#define REG_EBX 3
#define REG_ESP 4	/* stack pointer */
#define REG_EBP 5	/* frame/arg pointer */
#define REG_ESI 6
#define REG_EDI 7

/* segment registers for Context: */
#define SEG_ES	0
#define SEG_CS	2
#define SEG_SS	4
#define SEG_DS	6
#define SEG_FS	8
#define	SEG_GS	10

#endif /* __CONTEXT_H__ */

