/*  $Id: interrupt.c,v 1.1 1996/12/16 04:50:43 shaggy Exp shaggy $
 *      relative path: ./I386
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992,1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: interrupt.c,v $
 *  Revision 1.1  1996/12/16  04:50:43  shaggy
 *  Initial revision
 *
 *      creation date: 1992
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/global.h>
#include <stdio.h>
#include <error.h>
#include <vmem.h>

extern vaddr isr_table[];
extern intr isr_invalid;


extern void load_idt(IDTentry *adr, size_t size);


IDTentry idt[IDT_SIZE];

ushort irq_mask[16] = {
	0x0001,0x0002,0x0004,0xffff,
	0xffff,0x0020,0x0040,0x0080,
	0x0100,0x0200,0x0400,0x0800,
	0x1000,0x2000,0x4000,0x8000
};
			


void trap_svc(ulong type, ulong errcode, vaddr pc, vaddr va)
{
	if(type == TRAP_PGFAULT) {
		errcode &= 0xff;
		if(VM_FAULT_ACCV(errcode))
			vm_accv(0,0,pc,va,errcode&0xff);
		else
			vm_pagefault(0,0,pc,va,errcode&0xff);
		return;
	}
	panic(1,"%s; errcode=%x", trap_msg(type|(errcode&0xff)), errcode);
}


static void null_svc(int param, int irq)
{
}


void interrupt_init(void)
{
int i;
	kprintf("init_interrupts(): ");
	kprintf("idt=%8x ", idt);
	for(i=0; i<48; i++) {
		idt[i] = IDT(SEG_KERN_CODE,isr_table[i],SEGTYPE_KERNEL);
	}
/* only 48 interrupts now....
   i.e. -- no support for int 0x80 for Minix/Linux
   system calls... if people REALLY need this, let 
   me know  -- 
*/
	for(i=0; i<16; i++) 
		set_interrupt((vaddr)null_svc, 0, i);
	int_level = 0;
	load_idt(idt, sizeof(idt));
	kprintf("done\r\n");
}


void set_interrupt(vaddr func, int param, int irq)
{
	irq_call[irq] = func;
	irq_param[irq] = param;
}


