/*  mdmacro.h                                                   (CRC:29d08db7)
 *      relative path: ./I386
 *      creation date: 1992
 *      last modified: Wed Dec 18 07:15:55 1996
 *      header update: Wed Dec 18 07:15:56 1996
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __MDMACRO_H__
#define __MDMACRO_H__


#ifndef ASM

#include <port.h>

#define CRITICAL_BEGIN(ipl) __asm__("pushfl; popl %0; cli":"g="(ipl));
#define CRITICAL_END(ipl)   __asm__("pushl %0; popfl"::"g"(ipl));
#define LOCK(lock)	lock_wait((volatile lock_t*)&(lock));
#define GETLOCK(lock)	(lock_get((volatile lock_t*)&(lock)))
#define UNLOCK(lock)	lock_signal((volatile lock_t*)&(lock));
#define LOCK_CLEAR(lock)  (lock) = 0;
#define LOCK_FAST(lock)	__asm__("1: bts %0,$1; jcc 2f"::"m"(&lock)); \
			__asm__("push %0; call _wait_lock; jmp 1b"::"i"(&lock));\
			__asm__("2:");
#define UNLOCK_FAST(lock)	__asm__("btr %0,$1"::"m"(&lock));
#define CSWITCH_OFF(b)	{ (b) = cswitch_enable; cswitch_enable = FALSE; }
#define CSWITCH_ON(b) 	{ scheduler(); cswitch_enable = (b);  }

/* On Intel architectures, there is no way to invalidate a single entry,
 * only the entire cache... this might make page table modifications
 * more expensive than originally anticipated:
 */
#define TBIA()		__asm__("movl %%cr3,%0; movl %0,%%cr3"::"r"(0)); 
#define TBIS(va)	TBIA()
#define TBIR(adr,len)	TBIA()

#define SETPAGEDIR(pd)	__asm__("movl %0,%%cr3"::"r"((pd)));

/* port I/O: */

static __inline__ void out_portb(uchar v, ushort p) {
	__asm__("outb %%al,%%dx"::"a"(v),"d"(p)); }

static __inline__ void out_portw(ushort v, ushort p) {
	__asm__("outb %%al,%%dx; movb %%ah,%%al; \
	incw %%dx; outb %%al,%%dx"::"a"(v),"d"(p)); }

static __inline__ void out_regb(uchar r, uchar v, ushort p) {
	__asm__("outb %%al,%%dx; movb %0,%%al; \
	incw %%dx; outb %%al,%%dx"::"g"(v),"a"(r),"d"(p)); }

static __inline__ void out_regw(uchar r, ushort v, ushort p) {
	__asm__("movb %%al,%%ah; incb %%ah; outb %%al,%%dx; \
	movb %%ch,%%al; incw %%dx; outb %%al,%%dx; \
	movb %%ah,%%al; decw %%dx; outb %%al,%%dx; \
	movb %%cl,%%al; incw %%dx; outb %%al,%%dx" \
	::"c"(v),"a"(r),"d"(p)); }

static __inline__ uchar in_portb(ushort p) {
volatile uchar v;
	__asm__("inb %%dx,%%al":"a="(v):"d"(p));
	return((uchar)v); }

static __inline__ ushort in_portw(ushort p) {
volatile ushort v;
	__asm__("inw %%dx,%%ax":"a="(v):"d"(p));
	return((ushort)v); }

static __inline__ uchar in_regb(uchar r, ushort p) {
uchar v;
	__asm__("outb %%al,%%dx; incw %%dx; inb %%dx,%%al"\
		:"a="(v):"d"(p),"a"(r));
	return((uchar)v); }

static __inline__ ushort in_regw(uchar r, ushort p) {
volatile ushort v;
	__asm__("movb %%al,%%ah; incb %%ah; outb %%al,%%dx;\
		incw %%dx; inb %%dx,%%al; movb %%al,%%ch;\
		decw %%dx; movb %%ah,%%al; outb %%al,%%dx;\
		incw %%dx; inb %%dx,%%al; movb %%al,%%cl"\
		:"c="(v):"d"(p),"a"(r));
	return((ushort)v); }

static __inline__ ushort cmos_read(uchar a) {
	return(in_regb(a,CMOS_INDEX)); }

static __inline__ void cmos_write(uchar v, uchar a) {
	out_regb(a,v,CMOS_INDEX); }

#endif /* ASM */

#endif /* __MDMACRO_H__ */

