/*  mdparam.h                                                   (CRC:d3a9cb55)
 *      relative path: ./I386
 *      creation date: 1992
 *      last modified: Wed Dec 18 07:15:57 1996
 *      header update: Wed Dec 18 07:15:57 1996
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __MDPARAM_H__
#define __MDPARAM_H__


/* adjustable parameters: */
#define HZ_DEFAULT	200	/* 200 clock ticks per second */
#define TASKSIZE        (8*MEG)	/* maximum user process code/data size */
#define TEMPSIZE        (4*MEG)	/* maximum temporary region (per-thread) */
#define STACKSIZE       (4*MEG)	/* maximum size of stack for each thread */
#define KERNSIZE        (8*MEG)	/* maximum size of kernel */
#define KSTACK_SIZE     (8*K)   /* kernel stack size */

/* timing: */
#define SYS_HERTZ	14318180	/* 14.3Mhz system clock */
#define TIMER_HERTZ	1193182		/* 1.19Mhz timer clock */

/* memory: */
#define PAGE_SIZE	4096
#define NPTEPAGE	1024	/* number of PTE's per page directory */
#define MAXPTE		0x40000		/* maximum # of page table entries */

#define TASKBASE	0x00000000
#define STACKTOP	0x80000000
#define STACKBASE	(STACKTOP-MAXPTE*PAGE_SIZE)
#define TEMPBASE	(STACKTOP-STACKSIZE-TEMPSIZE)
#define KERNBASE	0xf0000000

/* interrupts: (defaults IRQ levels) */
#define INT_CLOCK	0		/* always */
#define INT_KB		1
#define INT_VSYNC	2
#define INT_SER1	3
#define INT_SER0	4				
#define INT_FD		5
#define INT_DISK	6
#define INT_PAR0	7
#define INT_RTIME	8
#define INT_CASCADE	9
#define INT_MOUSE	12
#define INT_COPROC	13
#define INT_HD		14

/* range checks: */
#if STACKSIZE%(4096*K)
#error STACKSIZE must be a multiple of 4096K
#endif
#if TEMPSIZE%(4096*K)
#error TEMPSIZE must be a multiple of 4096K
#endif
#if TASKSIZE%(4096*K)
#error TASKSIZE must be a multiple of 4096K
#endif
#if (KERNBASE+KERNSIZE)>=0xfffff000
#error KERNBASE+KERNSIZE must be at least one page less than 2^32
#endif

#endif /* __MDPARAM_H__ */

