/*  $Id: system.c,v 1.1 1996/11/12 01:41:00 shaggy Exp shaggy $
 *      relative path: ./I386
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: system.c,v $
 *  Revision 1.1  1996/11/12  01:41:00  shaggy
 *  Initial revision
 *
 *      creation date: 1992
 */

#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/global.h>
#include <error.h>
#include <port.h>
#include <keyboard.h>
#include <cmos.h>
#include <stdlib.h>
#include <boot/bootparam.h>
#include <boot/libsolo.h>


char *reg_name = "axcxdxbxspbpsidi";
char *seg_name = "ecsdfg";


void stack_traceback(Context *context, boolean sym);
void clock_set_hz(ushort hz);
ushort clock_get_hz(void);

extern BootParam *bootparam;


void system_init(void)
{
	cmos_write(4,CMOS_SHUTDOWN);	/* cause faults to hang for now */
}


void bootparam_print(void)
{
#ifdef OLD_BOOT_LOADER
	kprintf("bootparam: base/extmem=%dK,%dK crtc_adr=%x video_seg=%x scr=%dx%d,%d,%d\r\n",
		bootparam.memsize, bootparam.extmem, bootparam.crtc_adr, bootparam.video_seg,
		bootparam.scr_wid, bootparam.scr_hgt, 
		bootparam.cur_x, bootparam.cur_y);
	kprintf("a_text=%x a_data=%x a_bss=%x\r\n",
		bootparam.a_text, bootparam.a_data, bootparam.a_bss);
#else
	void *savemap;

	savemap = solo_MapBootParam();
	kprintf("bootparam: base/extmem=%dK,%dK\r\n",
		bootparam->base_size/1024, bootparam->ext_size/1024);
	kprintf("text=%x data=%x bss=%x\r\n",
		bootparam->text.vsize, bootparam->data.vsize, bootparam->bss.vsize);
	solo_UnmapBootParam(savemap);
#endif
}



ulong no_idt[2] = {0,0};


void reboot(void)
{
int i,z;
int cflag;
	kprintf("\nMy mind is going, Dave.  ");
	CRITICAL_BEGIN(cflag)
	cmos_write(0,CMOS_SHUTDOWN);		/* normal POST */
	vm_restore();
	*(ushort*)0x472 = 0x1234;	/* skip memory test */
	*(uchar*)0x412 = 0x0;
	z = 0;
	while(z++<10) {
		kprintf("I can feel it...  ");
		for(i=0; i<0x10000; i++)	
			if(!in_portb(KB_STATUS)&2)
				break;
		for(i=0; i<100000; i++)
			continue;
		out_portb(0xfe,KB_STATUS);
#ifdef __ELF__
		__asm__("lidt no_idt");
#else
		__asm__("lidt _no_idt");
#endif
	}
	kprintf("I feel much better now, Dave.\n");
	CRITICAL_END(cflag)
}


static void kernel_exit(int code)
{
	vm_restore();
	bootparam->exit(code);
}


void halt(void)
{
int k,oldk;
int cflag;
	while(1) {
		CRITICAL_BEGIN(cflag)
		kprintf("HALTING CPU... <r>eboot, e<x>it, <c>ontinue, <t>race, or <s>ymtrace: ");
		k = in_portb(KB_DATA);
		do {
			oldk = k;
			k = in_portb(KB_DATA);
		} while((k&0x80) || k == oldk);
		kprintf("\n");
		switch(k&0x7f) {
		case KEY_ESC:
		case KEY_R:
			reboot();
			break;
		case KEY_T:
			stack_traceback(NULL,FALSE);
			break;
		case KEY_S:		
			stack_traceback(NULL,TRUE);
			break;
		case KEY_C:
			CRITICAL_END(cflag)
			return;
		case KEY_X:
			kernel_exit(0);
		default:
			break;
		}
	}
}


uchar read_cmos_clock(int param)
{
	return(bcd_decode(cmos_read(param)));	
}


#ifdef XXX
uchar days_in_year[12] =
{0, 31, 31+28, 31+28+31, 31+28+31+30, 31+28+31+30+31,
 31+28+31+30+31+30, 31+28+31+30+31+30+31, 31+28+31+30+31+30+31+31,
 31+28+31+30+31+30+31+31+30, 31+28+31+30+31+30+31+31+30+31, 
 31+28+31+30+31+30+31+31+30+31+30};

void date_get(Date *date)
{
int n;
	date->sec = read_cmos_clock(WAT_SEC);
	date->min = read_cmos_clock(WAT_MIN);
	date->hour = read_cmos_clock(WAT_HOUR);
/*	date->dow = read_cmos_clock(WAT_DOW); */
	date->day = read_cmos_clock(WAT_DAY);
	date->month = read_cmos_clock(WAT_MON);
	date->year = read_cmos_clock(WAT_YEAR)
		 + 100*read_cmos_clock(WAT_CENTURY);
	n = (date->year - 1900) * 365
		+ (date->year-1901)/4 + days_in_year[date->month-1] 
		+ date->day;
	if((date->year % 4) == 0 && date->month > 2)
		n++;
	date->dow = (n % 7) + 1;
	/* day of week goes from 1-7 */
}
#endif



void context_print(void (*eprintf)(char *fmt,...), Context *context)
{	
int i;
	for(i=0; i<8; i++) {
		eprintf("e%2s=%8x ", &reg_name[i*2], context->reg[i]);
		if((i&3)==3)
			eprintf("\r\n");
	}
	for(i=0; i<6; i++) {
		eprintf("%cs=%4x ", seg_name[i], context->seg[i*2]);
	}
	eprintf("pc=%8x psl=%8x\r\n", context->pc, context->eflags);
	eprintf("pdbr=%8x ldt=%4x iomap=%4x link=%4x debug=%d\r\n",
		context->pdbr, context->ldt, context->iomap, context->link,
		context->debug);
	for(i=0; i<3; i++) 
		eprintf("sp%d=%4x:%8x ", i, 
			context->sp[i].ss, context->sp[i].esp);
	eprintf("\r\n");
}


vaddr pc_traceback(int level)
{
vaddr pc;
ulong *bp,*sp;
	pc = 0;
	__asm__("movl %%ebp,%0":"g="(bp));
	while(level-- >= 0) {
		if(!bp)
			break;
		sp = bp;		/* simulate 'leave' */
		bp = (ulong*)(*sp++);	/* simulate 'leave' */
		pc = (vaddr)(*sp++);	/* simulate 'ret' */
		if(!pc)
			break;
	}
	return(pc);
}

void stack_traceback(Context *context, boolean sym)
{
ulong *bp;
ulong *sp;
vaddr pc,newpc;
char buf[80];
int nparam;
	if(context) {
		pc = context->pc;
		sp = (ulong*)(context->reg[REG_ESP]);
		bp = (ulong*)(context->reg[REG_EBP]);
	} else {
		__asm__("movl %%esp,%0":"g="(sp));
		__asm__("movl %%ebp,%0":"g="(bp));
		pc = 0;
	}
	while(1) {
		if(pc) {
			if(sym && pc > KERNBASE && kinfo.Root) {
				kprintf("pc=%8x  ", pc); 
				kernsym_getbyvalue(pc, buf, sizeof(buf));
				kprintf("%s ", buf);
			} else
				kprintf("pc=%8x sp=%8x bp=%8x  ", pc, sp, bp); 
		}
		if(!bp) 
			break;
		sp = bp; 		 /* simulate 'leave' */
		bp = (ulong*)(*sp++);	 /* simulate 'leave' */
		newpc = (vaddr)(*sp++);	 /* simulate 'ret' */
		if(pc && newpc) {
			nparam = bp-sp;
			if(!bp)
				nparam = 1;
			if(nparam > 16)
				nparam = 16;
			/* check for 'addl $xx,%esp' */
			if(*(ushort*)(newpc) == 0xc483) 
				nparam = (*(uchar*)(newpc+2))/4;
			else if (*(uchar*)(newpc) != 0xc9)
				kprintf("?"); 
			kprintf("(");
			while(nparam--) {
				kprintf("%x",*sp++);	
				if(nparam)
					kprintf(",");
			}
			kprintf(")\n");	
		}
		if(!newpc || newpc == pc)
			break;
		pc = newpc;
	} 
	kprintf("\n");
}

