/*  $Id: video.c,v 1.1 1996/11/12 04:08:08 shaggy Exp shaggy $
 *      PC/AT basic video routines
 *      to be included from I386/console.c
 *      
 *      relative path: ./I386
 *  
 *       Intel 386 PC  machine dependent code
 *  
 *  Copyright (c) 1992,1996 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/*  $Log: video.c,v $
 *  Revision 1.1  1996/11/12  04:08:08  shaggy
 *  Initial revision
 *
 *      creation date: 1992
 */


#include "videoparam.h"
#include <boot/libsolo.h>


struct videoparam videoparam;

static int panic_lines;

static short *xpos_ptr, *ypos_ptr;

#define VT_SET_CURSOR(wid,x,y) \
	out_regw(0x0e,(x)+(y)*wid,videoparam.crtc_adr); 


void console_getxy(short *xptr, short *yptr) 
{
	*xptr = *xpos_ptr;
	*yptr = *ypos_ptr;
}

void console_setxyvars(short *xptr, short *yptr) 
{
	xpos_ptr = xptr;
	ypos_ptr = yptr;
}


void video_init(void)
{
	paddr vbase;
	int n;
	uchar *page0;		/* mapped to physical address 0 */

	/* Note that we can't just map in the BootParam structure to get
	 * at address 0, since the first page is not mapped,
	 * in order to trap null pointer references 
	 * Therefore, we must actually map memory in ourselves to get
	 * at that page.
	 */
	page0 = (uchar*)vm_map(&kernel.data,0,PAGE_SIZE, VM_KRW);

	if(page0[0x449] == 7)	/* if video mode = 7 (monochrome) */
		vbase = 0xb0000;	/* mono base address */
	else
		vbase = 0xb8000;	/* color base address */

	videoparam.crtc_adr = *(ushort*)(page0+0x463);
	videoparam.wid = page0[0x44a];
	videoparam.hgt = page0[0x484]+1;
	/*
	videoparam.xpos = page0[0x450];
	videoparam.ypos = page0[0x451];
	*/

	/* move BIOS cursor to bottom of screen */
	page0[0x450] = 0;
	page0[0x451] = videoparam.hgt-1;

	vm_free(&kernel.data, (vaddr)page0);	/* unmap page0 */

	videoparam.video_base = (short*)vm_map(&kernel.data,vbase,
		videoparam.wid*videoparam.hgt*2,VM_KRW);
	panic_lines = 0;

	/* we'll assume that the video cursor has the actual last 
	 * cursor position
	 */ 
	n = in_regw(0xe,videoparam.crtc_adr);
	videoparam.ypos = n/videoparam.wid;
	videoparam.xpos = n%videoparam.wid; 
	console_setxyvars(&videoparam.xpos,&videoparam.ypos);
}


static void fault_loop(void)
{
int cflag;
	CRITICAL_BEGIN(cflag)
	panic_event.event = 0;
	console_write("\007 Fault loop!\n",14);
	halt();
	/* theoretically, a user can
	 * choose to continue from halt().. */
	panic_lines = 0;
	CRITICAL_END(cflag)
}


int console_write(char *buf, int len)
{
char ch;
int x,n,i;
char *bp;
	videoparam.xpos = *xpos_ptr;
	videoparam.ypos = *ypos_ptr;
	i = len;
        while(i--) {
                ch = *buf++;
                switch(ch) {
                case '\b':
                        if (videoparam.xpos > 0)
                                videoparam.xpos--;
                        break;
                case '\t':
                        videoparam.xpos = (videoparam.xpos & ~7)+8;
                        break;
		case '\007':
			console_beep(400,100);
			break;
		case '\013':
			videoparam.xpos = 0;
			videoparam.ypos = 0;
			break;
/* XXX - removing for now, to prevent clearing critical data 
		case '\014':
			memset_w(videoparam.video_base, 0x700,
				videoparam.wid*videoparam.hgt);
			break;
*/
                case '\r':
                        videoparam.xpos = 0;
                        break;
                case '\n':
			videoparam.xpos = 0;
                        videoparam.ypos++;
			if (panic_event.event) {
				if (panic_lines++ >= videoparam.hgt-8) 
					fault_loop();
			}
                        break;
                default:
			videoparam.video_base[videoparam.ypos*videoparam.wid
					+videoparam.xpos]
				 = ch | 0x700;
                        if ((videoparam.xpos++) >= videoparam.wid) {
                                videoparam.xpos = 0;
                                videoparam.ypos++;
                        }
                }
                if (videoparam.ypos >= videoparam.hgt) {
			x = len;
			n = 1;
			bp = buf;
			while(x-- > 0 && n < videoparam.hgt) 
				if (*bp++ == '\n')
					n++;
                        videoparam.ypos = videoparam.hgt-n;
			memcpy(&videoparam.video_base[0],
			      &videoparam.video_base[n*videoparam.wid],
			      videoparam.wid*(videoparam.hgt-n)*sizeof(short));
			memset_w(&videoparam.video_base[videoparam.wid
				*(videoparam.hgt-n)], 0x700, videoparam.wid*n);
                }
        }
	*xpos_ptr = videoparam.xpos;
	*ypos_ptr = videoparam.ypos;
	VT_SET_CURSOR(videoparam.wid,videoparam.xpos,videoparam.ypos)
	return(len);
}



void console_beep(int freq, int delay)
{
int n;
int oldkb;
	n = TIMER_HERTZ/freq;
	/* set PIT2 (programmable interrupt timer) to frequency: */ 
	out_portb(0xb6,0x43);
	out_portb(n&0xff,0x42);
	out_portb(n>>8,0x42);
	/* activate speaker: */
	oldkb = in_portb(0x61);
	out_portb(oldkb | 3, 0x61);
	msleep(delay);
	/* turn it off: */
	out_portb(oldkb, 0x61);	
}


static heartbeat_flag = TRUE;

void heartbeat_toggle(void)
{
	heartbeat_flag = !heartbeat_flag;
}

void heartbeat(int i)
{
static uchar n[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int x;
	if(!heartbeat_flag)
		return;
	x = i&15;
	if ((n[x] = n[x]+1) >= 10)
		n[x] = 0;
	videoparam.video_base[videoparam.wid-17] = 0x730+'.';
	videoparam.video_base[videoparam.wid-16+x] = 0x730+n[x];
}

