/*  mdmacro.h                                                   (CRC:5bf1c5a7)
 *      relative path: ./VAX
 *      creation date: 1992
 *      last modified: Wed May 22 21:28:08 1996
 *      header update: Wed Dec 18 07:16:28 1996
 *  
 *       Digital VAX machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __MDMACRO_H__
#define __MDMACRO_H__


#ifndef ASM

#include <privreg.h>

#define CRITICAL_BEGIN(ipl) critical_begin(&ipl);
#define CRITICAL_END(ipl) critical_end(&ipl);

#define LOCK(lock)	lock_wait((volatile lock_t*)&(lock));
#define GETLOCK(lock)	(lock_get((volatile lock_t*)&(lock)))
#define UNLOCK(lock)	lock_signal((volatile lock_t*)&(lock));
#define LOCK_CLEAR(lock)  (lock) = 0;

#define CSWITCH_OFF(b)	{ (b) = cswitch_enable; cswitch_enable = FALSE; }
#define CSWITCH_ON(b) 	{ scheduler(); cswitch_enable = (b); }

#define TBIA()		mtpr(0,R_TBIA)
#define TBIS(va)	mtpr(va,R_TBIS)
#define TBIR(adr,len)	mtpr(0,R_TBIA)

/* register I/O: */

#define mtpr(v,r)	__asm__("mtpr %0,%1"::"g"(v),"i"(r))
#define mfpr(r,v)	__asm__("mfpr %1,%0":"=g"(v):"i"(r))


static __inline__ void critical_begin(int *ipl) {
	mfpr(R_IPL,*ipl); mtpr(IPL_MAX,R_IPL); }

static __inline__ void critical_end(int *ipl) {
	mtpr(*ipl,R_IPL); } 
 
static __inline__ int set_ipl(int ipl) {
volatile int n;
	mfpr(R_IPL,n); mtpr(ipl, R_IPL);
	return((int)n); }

static __inline__ int set_iplmin(int ipl) {
volatile int n;
	mfpr(R_IPL,n);
	if(ipl > n)
		 mtpr(ipl, R_IPL);
	return((int)n); }


#endif /* ASM */

#endif /* __MDMACRO_H__ */

