/*  system.c                                                    (CRC:2f12ccf1)
 *      relative path: ./VAX
 *      creation date: 1992
 *      last modified: Wed May 22 21:28:18 1996
 *      header update: Wed Dec 18 07:16:45 1996
 *  
 *       Digital VAX machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* system.c
 *  for VAX
 */

#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/global.h>
#include <error.h>
#include <keyboard.h>
#include <cmos.h>
#include <stdlib.h>


char *reg_name = "axcxdxbxspbpsidi";
char *seg_name = "ecsdfg";


void stack_traceback(Context *context, boolean sym);


void system_init(void)
{
#ifdef MVAX
	prom = (ulong*)vm_map(&kernel.data, 0x20040000,256*K, VM_KRW);
        vrom = (ushort*)vm_map(&kernel.data, 0x20140000,256*K, VM_KRW);
        nrom = (uchar*)vm_map(&kernel.data, 0x20100000,256*K, VM_KRW);
        erom = (uchar*)vm_map(&kernel.data, 0x20090000,128, VM_KRW); 
        nvram  = (ulong*)vm_map(&kernel.data, 0x200b0000,256, VM_KRW);
        nvram[CPMBX] = (nvram[CPMBX]&0xff33)|0xcc;  /* stop when halted */
#endif
}


void clock_set_interval(int hz)
{
	mtpr(0x40,R_ICCS);
}

uchar read_cmos_clock(int param)
{
	return(bcd_decode((nvram[param] >> 2) & 0xff));
}


void bootparam_print(void)
{
}


void reboot(void)
{
int cflag;
	kprintf("\nSyncing disks...  ");
	CRITICAL_BEGIN(cflag)
	nvram[CPMBX] = (nvram[CPMBX]&0xff33);	/* auto-reboot when halted */
	kprintf(" :) ");
	__asm__("halt");
	kprintf("I feel much better now, Dave.\n");
	CRITICAL_END(cflag)
}


static void fakeout(void)
{
int k,ch;
	kprintf("\n?06 HLT INST\n    PC = 800013E5\n>>> ");
	ch = 0;
	while(1) {
		while(!(k=console_readkey()));
		if(k == '\n' || k == '\r') {
			kprintf("\n\n") ;
			if (ch == '0') {
				for(k=0; k<2000000; k++)	
					continue;
				kprintf("?09 TIMEOUT\n>>> ");
			} else if (ch == 'c')
				return;
			else
				kprintf("?17 ILL CMD\n>>> ");
		} else
			kprintf("%c",k);
		ch = k;
	}
}


void halt(void)
{
int k;
int cflag;
	while(1) {
		CRITICAL_BEGIN(cflag)
		kprintf("HALTING CPU... <r>eboot, <c>ontinue, <t>race, or <s>ymtrace: ");
		while(!(k = console_readkey()));
		kprintf("\n");
		switch(k) {
		case '\033':
		case 'H':
			__asm__("halt");
			break;
		case 'h':
			fakeout();
			break;
		case 'r':
			reboot();
			break;
		case 't':
			stack_traceback(NULL,FALSE);
			break;
		case 's':
			stack_traceback(NULL,TRUE);
			break;
		case 'c':
			CRITICAL_END(cflag)
			return;
		}
	}
}


vaddr pc_traceback(int level)
{
vaddr pc;
ulong *fp;
	pc = 0;
	__asm__("movl %%fp,%0":"g="(fp));
	while(level-- >= 0 && fp) {
		pc = (vaddr)fp[4];
		fp = (ulong*)fp[3];
		if(!pc)
			break;
	}
	return(pc);
}

void stack_traceback(Context *context, boolean sym)
{
ulong *newfp;
ulong *fp;
ulong *ap;
vaddr pc;
char buf[80];
int nparam;
	if(context) {
		kprintf("Context traceback not supported\n");
		return;
	} else {
		__asm__("movl %%fp,%0":"g="(fp));
		pc = 0;
	}
	while(fp) {
		ap = (ulong*)fp[2];
		newfp = (ulong*)fp[3];
		pc = fp[4];
		if(sym)
			kprintf("pc=%8x  ", pc, ap, newfp); 
		else
			kprintf("pc=%8x ap=%8x fp=%8x  ", pc, ap, newfp); 
		if(sym && pc > KERNBASE && kinfo.Root) {
			kernsym_getbyvalue(pc, buf, sizeof(buf));
			kprintf("%s ", buf);
		}
		if(!ap) 
			break;
		fp = newfp;
		nparam = *ap++;
		kprintf("(");
		while(nparam--) {
			kprintf("%x",*ap++);	
			if(nparam)
				kprintf(",");
		}
		kprintf(")\n");	
	} 
	kprintf("\n");
}

