/*  vmem.c                                                      (CRC:bef5d66e)
 *      relative path: ./VAX
 *      creation date: 1992
 *      last modified: Wed May 22 21:28:47 1996
 *      header update: Wed Dec 18 07:16:52 1996
 *  
 *       Digital VAX machine dependent code
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* vmem.c
 *  virtual memory management 
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/global.h>
#include <error.h>
#include <stdio.h>
#include <privreg.h>

/* NOTE:  this memory management system 
 *        is limited to a physical size of 
 *        64K pages (32 meg, with 512byte pagesize)
 *        in order to conserve space on the tables
 *        that map available memory.
 *
 *        if it is rewritten for larger systems, 
 *        vm_refcount must first be changed to a ulong *
 *        and many other references must be changed appropriately
 */

static ulong vm_npages;
static ulong vm_nfreepages;
static ushort *vm_refcount;
static ushort vm_firstfree;
static lock_t pagelock;	  /* lock for available page freelist */


extern void etext(void),edata(void),end(void);	

/* routines common to 386 and VAX memory mapping: */
#include "../common/vmem_386_VAX.c"


#define RAMSIZE		4*MEG



static vaddr lomem = (vaddr)end;



void vm_init(void)
{	
int i, n;
ulong readadr;
extern vaddr lomem;	/* imported from sys/kmem.c */
ulong memsize;
extern vaddr scb;
	memsize = RAMSIZE;
	mem_init();
	LOCK_CLEAR(pagelock);
	vm_npages = memsize/(PAGE_SIZE);

	lomem = align(lomem,PAGE_SIZE);
	scb = lomem;		/* allocate 256 interrupt vectors */
	lomem += 0x400;

	/* set up page tables for kernel: */
	kernel.data.ptlen = KERNSIZE/PAGE_SIZE;
	kernel.data.pte = (ulong*)lomem;
	lomem += kernel.data.ptlen*sizeof(long);
	kernel.data.vbase = KERNBASE;

	vm_refcount = (ushort*)lomem;
	lomem += vm_npages*sizeof(ushort);
	memset(vm_refcount, 0, vm_npages*sizeof(ushort));

	vm_vmap_init(&kernel.data);

	/* set up free-list for physical pages: */
	vm_nfreepages = vm_npages;
	vm_firstfree = align(lomem-KERNBASE,PAGE_SIZE)/PAGE_SIZE;
	n = vm_firstfree;
	for(i=1; i<vm_npages-vm_firstfree; i++) { 
		vm_refcount[n] = i;
		n = i;
	}
	vm_refcount[n] = 0;

	/* actually map the kernel, with first segment read only */
	readadr = ((ulong)edata & ~(PAGE_SIZE-1));
	vm_map(&kernel.data, 0, lomem-KERNBASE, VM_KRW);
	/* make the code portion of it readable to user programs: */
	vm_setaccess(&kernel.data, KERNBASE, readadr-KERNBASE, VM_URKW);
	vm_kern_ptmap(NULL,&kernel.data);
	mtpr(0,R_P0BR);
	mtpr(0,R_P0LR);
	mtpr(0,R_P1BR);
	mtpr(0,R_P1LR);
	TBIA();
}


void vm_task_ptmap(Thread *node, Vmap *vmap)
{
	if(vmap) {
		node->context.p0br = (vaddr)vmap->pte;
		node->context.p0lr = vmap->ptlen;
	} else {
		node->context.p0br = 0;
		node->context.p0lr = 0;
	}
	if(node == thread) {
		mtpr(node->context.p0br, R_P0BR);
		mtpr(node->context.p0lr, R_P0LR);
	}
}


void vm_stack_temp_ptmap(Thread *node, Vmap *vm, Vmap *vmtemp)
{
	if(vm) {
		node->context.p1br = (vaddr)vm->pte;
		node->context.p1lr = MAXPTE-(vm->ptlen+vmtemp->ptlen);
	} else {
		node->context.p1br = 0;
		node->context.p1lr = 0;
	}
	if(node == thread) {
		mtpr(node->context.p1br, R_P1BR);
		mtpr(node->context.p1lr, R_P1LR);
	}
}


void vm_kern_ptmap(Thread *node, Vmap *vmap)
{
	mtpr((vaddr)vmap->pte, R_SBR);
	mtpr(vmap->ptlen, R_SLR);
}

