/*  parse.c                                                     (CRC:fe8a972f)
 *      relative path: ./clib
 *      creation date: 1992
 *      last modified: Wed May 22 21:31:47 1996
 *      header update: Wed Dec 18 07:16:57 1996
 *  
 *       C Library
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* parse.c
 *   routines to parse a string
 */



typedef struct parse_table {
	int	symbol;
	char	*string;
} ParseTable;



/* special characters in parse strings:
 *  \001   use the following character as the index to the new parse string
 *  \002   use the following index, any number of repetitions
 *         (prevents excessive recursion - use whenever possible)
 */


int parse(table, index, str, pos)
ParseTable *table;
int index;
char *str;
int pos;
{
int n;
char *sp;
/*	printf("parse(%x,%d,%s,%d,%x)\n",table,index,str,pos); */
	if(table[index].symbol)
		printf("sym=%d\n",table[index].symbol);
	sp = table[index].string;
	while(*sp && *sp != ':') {
		if(str[pos] >= sp[0] && str[pos] <= sp[1])
			return(pos+1);
		sp += 2;
	}
	while(*sp) {
		if(*sp == '\002') {
			int spos = pos;
			while((n=parse(table,(int)sp[1],str,pos))>pos)
				pos = n;
			if(pos > spos)
				return(pos);
		} else if(*sp == '\001') {
			sp++;
			n = parse(table,(int)(*sp),str,pos);
			if(n>pos)
				return(n);
		} else {
			if(str[pos] == *sp)
				return(pos+1);
		}
		sp++;
	}
	return(pos);
}




/* for testing: */
main()
{
static ParseTable pt[] = {
	/* 000:main */		{0,":\001\010"},
	/* 001:alpha_cap */	{0,"AZ"},
	/* 002:alpha_lower */	{0,"az"},
	/* 003:numeric */	{0,"09"},
	/* 004:alphanumeric */	{0,"AZaz09:$_"},
	/* 005:operator */	{0,":~!%^&+-*/,\001\007"},
	/* 006:whitespace */	{0,": \t\n\r"},
	/* 007:special op */	{0,"++"},
	/* 010:symbol */	{1,":\002\004\002\006\001\005\001\005"}
};
char str[256];
char buf[256];
int pos;
int n;
	while(1) {
		gets(str);
		pos = 0;
		while((n = parse(pt,0,str,pos)) > pos) {
			strncpy(buf,str+pos,n-pos);
			buf[n-pos] = '\0';
			printf("%d,%d:(%s)\n",n,pos,buf);
			pos = n;
		}	
		if (n < strlen(str)) {
			for(pos=0; pos<n; pos++)
				printf(" ");
			printf("^ syntax error\n");
		}
	}
}

