/*  file.c                                                      (CRC:19850041)
 *      simple file and stream objects
 *
 *      relative path: ./drv
 *      creation date: 1992
 *      last modified: Thu Dec 19 18:15:55 1996
 *      header update: Fri Dec 20 09:09:05 1996
 *  
 *       Device drivers
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <object.h>
#include <error.h>
#include <sys/global.h>
#include <stdio.h>
#include <dir.h>


#define LBLOCKSIZE	4096
#define SBLOCKSIZE	64
#define FSIZE0		32
#define FSIZE1		(FSIZE0+LBLOCKSIZE)
#define FSIZE2		(FSIZE1+LBLOCKSIZE)
#define FSIZE3		(FSIZE2+1024*LBLOCKSIZE)
#define FSIZE4		(FSIZE3+1024*1024*LBLOCKSIZE)
/* FSIZE4 is maximum file size --> over 4 gigs */

typedef struct file {
	ulong	size;
	ulong	atime;
	ulong	mtime;
	ulong	ctime;
	vaddr	d0;
	vaddr	d1;	
	vaddr	i1;
	vaddr	i2;
	char	header[32];
} File;


extern obj_t _size, _atime, _ctime, _mtime;



static vaddr fsb_alloc(void)
{
	return(vm_alloc(&kernel.data,PAGE_SIZE,VM_KRW));
}

static void fsb_free(vaddr adr)
{
	vm_free(&kernel.data, adr);
}


static vaddr fs_getblock(File *f, ulong index)
{
ulong *v;
	if(index <= FSIZE1)
		return(f->d0);
	if(index <= FSIZE2)
		return(f->d1);
	if(index <= FSIZE3) {
		v = (ulong*)(f->i1);
		if(v) 
			return(v[(index-FSIZE2)/LBLOCKSIZE]);
		return(0);
	}
	v = (ulong*)(f->i2);
	if(v) {
		v = (ulong*)v[(index-FSIZE3)/(LBLOCKSIZE*1024)];
		if(v)
			return(v[((index-FSIZE3)/LBLOCKSIZE)&0x3ff]);
	}
	return(0);
}

static vaddr fs_allocblock(File *f, ulong index)
{
ulong *v, *i;
	if(index <= FSIZE1) {
		if(!(v = (ulong*)(f->d0)))
			v = (ulong*)(f->d0 = fsb_alloc());
		return((vaddr)v);
	}
	if(index <= FSIZE2) {
		if(!(v = (ulong*)(f->d1)))
			v = (ulong*)(f->d1 = fsb_alloc());
		return((vaddr)v);
	}
	if(index <= FSIZE3) {
		if(!(v = (ulong*)(f->i1)))
			v = (ulong*)(f->i1 = fsb_alloc());
		i = &v[(index-FSIZE2)/LBLOCKSIZE];
		if(!(v = (ulong*)(*i)))
			v = (ulong*)(*i = fsb_alloc());
		return((vaddr)v);
	}
	if(!(v = (ulong*)(f->i2)))
		v = (ulong*)(f->i2 = fsb_alloc());
	i = &v[(index-FSIZE3)/(LBLOCKSIZE*1024)];
	if(!(v = (ulong*)(*i)))
		v = (ulong*)(*i = fsb_alloc());
	i = &v[((index-FSIZE3)/LBLOCKSIZE)%1024];
	if(!(v = (ulong*)(*i)))
		v = (ulong*)(*i = fsb_alloc());
	return((vaddr)v);
}



static obj_t file_init(ObjFrame *frame, File *file)
{
	frame->data = file = (File*)(KMALLOC(File));
	frame->size = sizeof(File);
	file->size = 0;
	file->atime = kinfo.systime;
	file->mtime = kinfo.systime;
	file->ctime = kinfo.systime;
	file->d0 = 0;
	file->d1 = 0;
	file->i1 = 0;
	file->i2 = 0;
	return(OFLAG_HASVEC|OFLAG_KERNEL);
}

static obj_t file_free(ObjFrame *frame, File *f)
{
int i,n;
ulong *v;
	if(f->d0)
		fsb_free(f->d0);
	if(f->d1)
		fsb_free(f->d1);
	if(f->i1) {
		for(i=0; i<1024; i++)
			if(((ulong*)(f->i1))[i])
				fsb_free(((ulong*)(f->i1))[i]);
		fsb_free(f->i1);
	}
	if(f->i2) {
	    for(i=0; i<1024; i++)
		if(((ulong*)(f->i2))[i]) {
		    v = (ulong*)(((ulong*)(f->i2))[i]);
		    for(n=0; n<1024; n++)
			if(v[n])
			    fsb_free(v[n]);
		    fsb_free((vaddr)v);
		}
	    fsb_free(f->i2);		
	}
	KFREE(f);
	return(OK);
}


static obj_t file_readat(ObjFrame *frame, File *f, 
			uchar *buf, int len, int index)
{
int i,n;
vaddr v;
	if(!len)
		return(0);
	n = 0;
	while(index < f->size && n < len) {
		v = 0;
		if(index < FSIZE0) {
			i = min(FSIZE0-index,len-n);
			memcpy(buf,&f->header[index],i);
			buf += i;
			index += i;
			n += i;
			continue;
		}
		v = fs_getblock(f, index);
		i = ((index-FSIZE0-1)&(LBLOCKSIZE-1))+1;
		i = min(i,len-n);
		if(v) 
			memcpy(buf,(void*)(v+((index-FSIZE0)&(LBLOCKSIZE-1))),i);
		else
			memset(buf,0,i);
		buf += i;
		index += i;
		n += i;
	}
	return(n);
}


static obj_t file_writeat(ObjFrame *frame, File *f, 
			uchar *buf, int len, int index)
{
int i, n;
vaddr v;
	if(!len)
		return(0);
	n = 0;
	while(n < len) {
		v = 0;
		if(index < FSIZE0) {
			i = min(FSIZE0-index,len-n);
			memcpy(&f->header[index],buf,i);
			buf += i;
			index += i;
			n += i;
			continue;
		}
		v = fs_allocblock(f, index);
		i = ((index-FSIZE0-1)&(LBLOCKSIZE-1))+1;
		i = min(i,len-n);
		memcpy((void*)(v+((index-FSIZE0)&(LBLOCKSIZE-1))),buf,i);
		buf += i;
		index += i;
		n += i;
	}
	if(index+n > f->size)
		f->size = index+n;
	return(n);
}


static MethodTable file_method[] = {
	{&_init,(obj_method)file_init},
	{&_free,(obj_method)file_free},
	{&_readat,(obj_method)file_readat},
	{&_writeat,(obj_method)file_writeat}};

static AttrTable file_attr[] = {
	{&_size,offsetof(File,size),ATYPE_ULONG},
	{&_atime,offsetof(File,atime),ATYPE_ULONG},
	{&_ctime,offsetof(File,ctime),ATYPE_ULONG},
	{&_mtime,offsetof(File,mtime),ATYPE_ULONG}};

static ObjClass Class_File = {
	file_method,
	file_attr,
	arraysize(file_method),
	arraysize(file_attr)};




/* XXX - should eventually have built-in buffering */

typedef struct stream {
	ulong	pos;
	obj_t	fd;
} Stream;

extern label_t _pos;


static obj_t stream_init(ObjFrame *frame, Stream *stream, obj_t fd)
{
	stream = frame->data = (Stream*)(KMALLOC(Stream));
	frame->size = sizeof(Stream);
	stream->pos = 0;
	stream->fd = fd;
	return(OFLAG_HASVEC|OFLAG_KERNEL);
}


static obj_t stream_read(ObjFrame *frame, Stream *stream, obj_t buf, int len)
{
obj_t rv;
	rv = err_(stream->fd, _readat, buf, len, stream->pos);
	stream->pos += len;
	return(rv);
}


static obj_t stream_write(ObjFrame *frame, Stream *stream, obj_t buf, int len)
{
obj_t rv;
	rv = err_(stream->fd, _writeat, buf, len, stream->pos);
	stream->pos += len;
	return(rv);
}


static MethodTable stream_method[] = {
	{&_init,(obj_method)stream_init},
	{&_read,(obj_method)stream_read},
	{&_write,(obj_method)stream_write}};

static AttrTable stream_attr[] = {
	{&_pos,offsetof(Stream,pos),ATYPE_ULONG}};

static ObjClass Class_Stream = {
	stream_method,
	stream_attr,
	arraysize(stream_method),
	arraysize(stream_attr)};


obj_t Main(ObjFrame *frame, vaddr base, obj_t arg, obj_t env)
{
obj_t class_file, class_stream, class_object;
	cprintf("File/Stream Objects  v0.01  Nov 1992\n");
	class_object = dir_parse(OBJ_ROOT, "/class/Object");
	class_file = _(class_object, _class, Ptr(&Class_File), ___, ___);
	class_stream = _(class_object, _class, Ptr(&Class_Stream), ___, ___);
	dir_create(OBJ_ROOT,"/class/File", class_file);
	dir_create(OBJ_ROOT,"/class/Stream", class_stream);
	return(OK);
}

