/*  serial.c                                                    (CRC:d162b918)
 *      Serial I/O for PC's
 *
 *      relative path: ./drv
 *      creation date: 1992
 *      last modified: Thu Dec 19 18:18:04 1996
 *      header update: Fri Dec 20 09:09:10 1996
 *  
 *       Device drivers
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <error.h>
#include <stdio.h>
#include <dir.h>


#define SER_BUF_MAX	2044


typedef struct serial_buf {
	ushort	head;
	ushort	tail;
	char	buf[SER_BUF_MAX];
} SerialBuf;



extern obj_t _param, _data, _speed, _break, _hangup;
extern obj_t _stdin, _stdout, _wid, _hgt, _rnoblock;


static __inline__ int serbuf_readchar(SerialBuf *buf)
/* returns -1 if no characters can be read */
{
register int n;
char ch;
	if((n = buf->head) == buf->tail)
		return(-1);
	ch = buf->buf[n++];
	if(n >= SER_BUF_MAX)
		n = 0;
	buf->head = n;
	return(ch);
}


static __inline__ int serbuf_writechar(SerialBuf *buf, char ch)
/* returns -1 if no characters can be written */
{
register int n;
	n = buf->tail;
	buf->buf[n++] = ch;
	if(n >= SER_BUF_MAX)
		n = 0;
	if(n == buf->head)
		return(-1);
	buf->tail = n;
	return(0);
}




#include <mdserial.c>


static void serbuf_scan(void)
{
int port;
boolean active;
char buf[2048];
int n,i;
	port = 0;
	active = TRUE;
	while(1) {
		if(++port >= NSERIAL) {
			port = 0;
			if(!active) 
				_(OBJ_THREAD, _msleep, 0, ___, ___);
			active = FALSE;
		} 
		if(!ser_idata[port])
			continue;
		n = 0;
		while((buf[n] = serbuf_readchar(ser_idata[port])) >= 0) {
			n++;
			if(n >= sizeof(buf))
				break;
		}
		if(n > 0) {
			_(ser_tty[port], _devwrite, Ptr(buf), n, ___);
			active = TRUE;
		}
		n = _(ser_tty[port], _devread, Ptr(buf), sizeof(buf), ___);
		for(i=0; i<n; i++) {
			while(serbuf_writechar(ser_odata[port], buf[i]) < 0) {
				ser_write(port); 
				_(OBJ_THREAD, _msleep, 0, ___, ___);
			}
			active = TRUE;
		}
		if(n > 0) 
			ser_write(port);
	}
}


static obj_t serial_init(ObjFrame *frame, void *data, int id)
{
	frame->data = (void*)id;
	frame->size = 0;
	return(OFLAG_KERNEL);
}


static MethodTable serial_method[] = {
	{&_init,(obj_method)serial_init},
	{&_break,(obj_method)serial_break},
	{&_hangup,(obj_method)serial_hangup}};

static ObjClass Class_Serial = {
	serial_method,
	NULLP(AttrTable),
	arraysize(serial_method),
	0};


obj_t Main(ObjFrame *frame, vaddr base, obj_t arg, obj_t env)
{
int i;
char buf[40],msg[40];
obj_t class_buffer;
obj_t class_object;
obj_t class_tty;
obj_t class_serial;
obj_t ser_ibuf, ser_obuf;
obj_t ser;

	cprintf("Serial I/O Driver v0.02  Nov 1992\n");

	mdserial_init();

	class_object = dir_parse(OBJ_ROOT,"/class/Object");
	class_tty = dir_parse(OBJ_ROOT,"/class/TTY");
	class_buffer = dir_parse(OBJ_ROOT,"/class/Buffer");
	class_serial = _(class_object, _class, Ptr(&Class_Serial), ___, ___);
	for(i=0; i<NSERIAL; i++)
	    if(serial_init_port(i,msg)) {
		ser_ibuf = _(class_buffer, _new, ___, ___, ___);
		ser_obuf = _(class_buffer, _new, ___, ___, ___);
		_(ser_obuf, _set, _rnoblock, OBJ_TRUE, ___);
		ser_idata[i] = KMALLOC(SerialBuf);
		ser_idata[i]->head = 0;
		ser_idata[i]->tail = 0;
		ser_odata[i] = KMALLOC(SerialBuf);
		ser_odata[i]->head = 0;
		ser_odata[i]->tail = 0;
		ser_tty[i] = _(class_tty, _new, ___, ___, ___);
		ser = _(class_serial, _new, i, ___, ___);
		_(ser_tty[i], _next, ser, ___, ___);
		_(ser_tty[i], _set, _stdin, ser_ibuf, ___);
		_(ser_tty[i], _set, _stdout, ser_obuf, ___);
		_(ser_tty[i], _set, _wid, 80, ___);
		_(ser_tty[i], _set, _hgt, 24, ___);
		sprintf(buf,"/dev/%s", devname[i]);
		dir_create(OBJ_ROOT,buf,ser_tty[i]);
		serial_setline(i, devspeed[i], devparam[i]);
		cprintf("\t%s: %5d %s  %s\n", buf, 
			devspeed[i], devparam[i], msg); 
	    } else {
		ser_idata[i] = NULL;
		ser_odata[i] = NULL;
	    }
	_(OBJ_THREAD, _new, Label("serial"), Ptr(serbuf_scan), 0); 
	return(OK);
}

