/*  error.h                                                     (CRC:ae7b5504)
 *      relative path: ./include
 *      creation date: 1992
 *      last modified: Wed May 22 21:33:30 1996
 *      header update: Wed Dec 18 07:17:19 1996
 *  
 *       Include files
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __ERROR_H__
#define __ERROR_H__

#include <sys/types.h>
#include <object.h>

#define OK		OBJ_NULL	/* no error */

#define ERR_FAILURE	ObjErr(0x00)	/* general failure */

/* params: */
#define ERR_INVPARAM	ObjErr(0x10)	/* invalid parameter */
#define ERR_INVTYPE	ObjErr(0x11)	/* invalid parameter type */

/* files: */
#define ERR_NOFILE	ObjErr(0x20)	/* file not found */
#define ERR_INVFILE	ObjErr(0x21)	/* invalid file format */
#define ERR_SHORTFILE	ObjErr(0x22)	/* unexpected EOF */
#define ERR_EOF		ObjErr(0x23)	/* end of file */

/* buffers: */
#define ERR_EOB		ObjErr(0x30)	/* end of buffer */

/* memory: */
#define ERR_NOMEM	ObjErr(0x40)	/* not enough memory */
#define ERR_NOBUF	ObjErr(0x41)	/* out of buffers */

/* object-oriented: */
#define ERR_NOMETHOD	ObjErr(0x50)	/* no such method */
#define ERR_NOATTR	ObjErr(0x51)	/* no such attribute */
#define ERR_EXISTATTR	ObjErr(0x52)	/* attribute already exists */
#define ERR_EOA		ObjErr(0x53)	/* end of attributes */
#define ERR_INVOBJ	ObjErr(0x54)	/* invalid object */
#define ERR_NOREAD	ObjErr(0x55)	/* no read access */
#define ERR_NOWRITE	ObjErr(0x56)	/* no write access */

/* threads: */
#define ERR_NOTHREAD	ObjErr(0x60)	/* no such thread id */

/* tasks: */
#define ERR_NOTASK	ObjErr(0x70)	/* no current task */

#endif  /* __ERROR_H__ */

