/*  stdio.h                                                     (CRC:a3653eb8)
 *      relative path: ./include
 *      creation date: 1992
 *      last modified: Wed May 22 21:35:53 1996
 *      header update: Wed Dec 18 07:17:33 1996
 *  
 *       Include files
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __STDIO_H__
#define __STDIO_H__

#include <sys/types.h>
#include <stddef.h>
#include <error.h>
#include <fcntl.h>
#include <stdarg.h>

typedef ulong fpos_t;

typedef struct {
	obj_t	fd;
	fpos_t	pos;
} FILE;

extern FILE _stdfile[];

#define EOF	ERR_EOF
#define stdin	&_stdfile[0]
#define stdout	&_stdfile[1]
#define stderr	&_stdfile[2]

#define BUFSIZ		XXX
#define FILENAME_MAX	MAXLABEL
#define FOPEN_MAX	MAXUSHORT
#define L_tmpnam	XXX
#define TMP_MAX		XXX
#define _IOFBF		XXX
#define _IOLBF		XXX
#define _IONBF		XXX

extern int voprintf(obj_t oid, char *fmt, va_list args);
extern int oprintf(obj_t oid, char *fmt, ...);
extern int cprintf(char *fmt, ...);
extern int printf(char *fmt, ...);
extern int kprintf(char *fmt, ...);
extern int vkprintf(char *fmt, va_list args);
extern int eprintf(char *fmt, ...);
extern int vprintf(char *fmt, va_list args);
extern int vsprintf(char *s, char *fmt, va_list args);
extern int sprintf(char *s, char *fmt, ...);
extern int puts(char *s);
extern int putchar(char ch);
extern int getchar(void);
extern char *ogets(char *s, int len, obj_t oid);
/* char *gets(char *str);  
 * XXX - gets() is a major security hole, because it has no
 * way to specify the buffer size , therefore it has been
 * removed.  Use getstrn(buf,len) instead  */
extern char *getstrn(char *s, int len);

extern int fclose(FILE *stream);
extern FILE *fdopen(int fildes, const char *type);
extern int feof(FILE *stream);
extern int ferror(FILE *stream);
extern int fflush(FILE *stream);
extern int fgetc(FILE *stream);
extern int fgetpos(FILE *stream, fpos_t *pos);
extern char *fgets(char *s, int n, FILE *stream);
extern int fileno(FILE *stream);
extern FILE *fopen(const char *filename, const char *mode);
extern int fprintf(FILE *stream, const char *fmt, ...);
extern int fputc(int c, FILE *stream);
extern int putc(int c, FILE *stream);
extern int fputs(const char *s, FILE *stream);
extern size_t fread(void *ptr, size_t size, size_t nelem, FILE *stream);
extern FILE *freopen(const char *filename, const char *mode, FILE *stream);
extern int fscanf(FILE *stream, const char *format, ...);
extern int fseek(FILE *stream, long offset, int whence);
extern int fsetpos(FILE *stream, const fpos_t *pos);
extern long ftell(FILE *stream);
extern size_t fwrite(const void *ptr, size_t size, size_t nelem, FILE *stream);

extern void perror(const char *msg);
extern void print_error(const char *msg, ulong errcode);

#endif /*__STDIO_H__ */
