/*  resource.h                                                  (CRC:e733d4d5)
 *      relative path: ./include/sys
 *      creation date: 1992
 *      last modified: Tue Dec 10 14:29:41 1996
 *      header update: Wed Dec 18 07:17:43 1996
 *  
 *       Include files
 *        System Include files
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#ifndef __RESOURCE_H__
#define __RESOURCE_H__


#include <sys/types.h>
#include <stddef.h>
#include <object.h>
#include <sys/param.h>
#include <string.h>
#include <macro.h>
#include <mdresource.h>


/*** Queue structures ***/

typedef struct queue {
	struct queue *next;
	struct queue *prev;
} Queue;


typedef void *ZoneP;


/*** Events ***/

typedef struct event {
	struct event *next;
	struct event *prev;
        ulong   event;          /* event code */
        ulong   param;          /* parameter code */
        obj_t   sender;         /* thread id of sender */
	obj_t	receiver;	/* thread id of receiver */
	ulong	time;		/* time to send event or time sent */
        ulong   aux1;		/* auxilliary parameters */
        ulong   aux2;
        ulong   aux3;
        ulong   aux4;
} Event;



#ifdef KERNEL

/*** Task classes ***/


typedef struct memlist {
	struct memlist *next;
	struct memlist *prev;
	struct memlist *free;
	ulong	size;
} MemList;


/*** Tasks ***/

typedef struct task {
	struct task *next;	/* queue links */
	struct task *prev;
	Vmap	data;		/* code/data address map */
	obj_t	self;		/* task object id */
	Queue	mem;		/* memory allocation queue */
	MemList *memfree;	/* first free memory block */
	Object  *objtable;	/* array of task's objects */
	long	objtable_size;	/* size of object array */
	long	objtable_free;	/* first free object */
	ushort	ref;		/* task reference count */
	lock_t	objlock;	/* lock for object table access */
	lock_t	memlock;	/* lock for memory queue access */
} Task;


/*** Threads ***/

typedef struct thread {
	struct thread *next;
	struct thread *prev;
	obj_t	self;
	obj_t	parent;
	obj_t	name;
	obj_t	std_obj[NBUFDESC];
	obj_t	cwd;		/* current working directory */
	obj_t	root;		/* root directory */
	obj_t	arg;		/* calling arguments */
	obj_t	env;		/* environment */
	obj_t	path;		/* search path */
	Queue	eventq;
	Vmap	stack;		/* kernel and user stack */
	Vmap	temp;		/* temporary shared memory */
	Task	*task;
	vaddr	ustack;		/* top of user stack */
	vaddr	kstack;		/* top of kernel stack */
	ulong   w_event;	/* event mask to wait for */
	ulong	w_param;	/* parameter to wait for */
	Context context;
	ulong	stime;		/* start time */
	ulong	rtime;		/* amount of run-time */
	uchar	quantum;
	uchar	max_quantum;
	boolean	cswitch_enable;	/* context switching enable */
} Thread;



/*** file system simulation ***/

typedef struct static_fs {
	char	*name;
	ulong	size;
	ulong	ofs;
	uchar	*data;
} StaticFS;



/* kernel stats and parameters: */

typedef struct kern_info {
	obj_t	Root;
	/* scheduling info: */
	ulong	systime;
	ushort	hz;
	ushort	minquantum;
	ushort	maxquantum;
	ushort	quantum_wait;
	ushort	quantum_sleep;
	ushort	quantum_cpubound;
	ushort	tload1;
	ushort	tload10;
	ushort	tload60;
	ushort	tcount;
	ulong	tcount1;
	ulong	tcount10;
	ulong	tcount60;
	ushort	time1;
	uchar	time10;
	uchar	time60;
	/* memory stats: */
	int	wasted_bytes;
} KernInfo;



/* zone.c: */
extern ZoneP zinit(int size, char *name);
extern void zaddmem(ZoneP zone, vaddr adr, int size);
extern void *zalloc(ZoneP zone);
extern void zfree(ZoneP zone, void *elem);
extern void zone_status(void);

/* vmem.c: */
extern void vm_init(void);
extern vaddr vm_alloc(Vmap *vm, int len, ulong accmode);
extern vaddr vm_realloc(Vmap *vm, vaddr adr, int len);
extern vaddr vm_reserve(Vmap *vm, int len);
extern vaddr vm_map(Vmap *vm, paddr phys, int len, ulong accmode);
extern vaddr vm_map_join(Vmap *vm, paddr phys, int len, ulong accmode);
extern vaddr vm_share(Vmap *vm, Vmap *vmsrc, vaddr adr, int len, ulong accmode);
extern void vm_free(Vmap *vm, vaddr adr);
extern vaddr vm_temp_alloc(int len, ulong accmode);
extern vaddr vm_temp_map(paddr adr, int len, ulong accmode);
extern vaddr vm_temp_share(Vmap *vmsrc, vaddr adr, int len, ulong accmode);
extern void vm_temp_free(vaddr adr);
extern vaddr vm_stack_alloc(Vmap *vm, int len, ulong accmode);
extern void vm_setaccess(Vmap *vm, vaddr adr, int len, ulong accmode);
extern void vm_copyto(Vmap *vm, vaddr dest, vaddr src, int len);
extern void vm_copyfrom(Vmap *vm, vaddr src, vaddr dest, int len);
extern void vm_task_open(Vmap *vm);
extern void vm_stack_temp_open(Vmap *vm, Vmap *vmtemp);
extern void vm_task_close(Vmap *vm);
extern void vm_stack_temp_close(Vmap *vm, Vmap *vmtemp);
extern Vmap *vm_getmap(vaddr adr);
extern void vm_status(void);
extern void vm_list(void);
extern void vm_task_ptmap(Thread *thread, Vmap *vmap);
extern void vm_stack_temp_ptmap(Thread *thread, Vmap *vm, Vmap *vmtemp);
extern void vm_kern_ptmap(Thread *thread, Vmap *vmap);
extern ulong vm_getpsize(Vmap *vm, ulong shared);
extern paddr vm_getphys(vaddr adr);


/* task.c: */
extern obj_t exec_obj(obj_t fd, int argc, char **argv);
extern obj_t kern_exec_obj(obj_t fd, int argc, char **argv);
extern status_t sym_getbyvalue(obj_t fd, vaddr val, char *buf, int buflen);
extern int sym_getbyname(obj_t fd, char *buf);
extern status_t kernsym_getbyvalue(int val, char *buf, int buflen);
extern int kernsym_getbyname(char *buf);
extern void sym_register(obj_t fd, obj_t wd);


/* kmem.c: */
extern void mem_init(void);
extern void mem_list(void);
extern void mem_status(void);
extern void *ksmalloc(int size);		/* static malloc */
extern void ksfree(void *adr, int size);	/* static free */
extern void *kmalloc(int size);
extern void *ksmalloc(int size);
extern void *kcalloc(int elem, int size);
extern void *krealloc(void *adr, int size);
extern void kfree(void *adr);
extern void *task_mem_malloc(Task *task, int size);
extern void task_mem_free(Task *task, void *adr);


/* thread.c: */
extern void thread_next(void);
extern void thread_settask(Thread *node, Task *task);


/* event.c: */
extern void event_init(void);
extern Event *event_recv(ulong evmask, ulong param);
extern status_t event_send(Event *event);
extern ulong waitfor(ulong evmask, ulong param);
extern void signalall(ulong evmask, ulong param);
extern void lock_wait(volatile lock_t *lock);
extern int lock_get(volatile lock_t *lock);
extern void lock_signal(volatile lock_t *lock);


/* trap.c: */
extern void panic(vaddr pc, char *fmt, ...);
extern char *trap_msg(ulong type);


/* queue.c: */
extern void qclear(Queue *queue);
extern boolean qempty(Queue *queue);
extern void qaddhead(Queue *queue, Queue *new);
extern void qaddtail(Queue *queue, Queue *new);
extern Queue *qremnode(Queue *queue);
extern Queue *qremhead(Queue *queue);
extern Queue *qremtail(Queue *queue);
extern Queue *qnext(Queue *queue);


/* timer.c: */
extern void scheduler(void);
extern void msleep(int msec);
extern void clock_init(void);
extern void schedule_event(Event *event);


/* register.c: */
extern void dir_register(void);


/* name.c: */
extern void Name_init(void);


/* console.c: */
extern int kprintf(char *fmt, ...);


/* system.c: */
extern ushort clock_get_hz(void);
extern void clock_set_hz(ushort hz);


/* object initializations: */
extern void Object_init(void);
extern void Buffer_init(void);
extern void Dir_init(void);
extern void Thread_init(void);
extern void Task_init(void);
extern void StaticFile_init(void);


#endif /* KERNEL */


/* misc: */
extern void nuke_fs(void);
extern void load_average(void);
extern void sys_stat(void);


#endif  /* __RESOURCE_H__ */

