/*  stdlib.c                                                    (CRC:dcd87f71)
 *      relative path: ./posix
 *      creation date: 1992
 *      last modified: Wed May 22 21:37:04 1996
 *      header update: Wed Dec 18 07:18:00 1996
 *  
 *       POSIX standard library
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

#include <sys/types.h>
#include <stdlib.h>
#include <object.h>
#include <string.h>
#include <setjmp.h>
#include <stdio.h>

static ulong seed = 0;

int rnd(int max)
{
	seed = seed*1103515245+12345;
	return((int)((seed/65536)%max));
}

void srand(ulong seed)
{
	seed = seed;
}

int rand(void)
{
	return(rnd(RAND_MAX));
}


void *malloc(size_t size)
{
	return((void*)_(OBJ_TASK, _mem_alloc, size, ___, ___));
}

void *calloc(size_t nelem, size_t size)
{
void *adr;
	adr = (void*)_(OBJ_TASK, _mem_alloc, size*nelem, ___, ___);
	memset(adr,0,size*nelem);
	return(adr);
}

void *realloc(void *ptr, size_t size)
{
void *newptr;
	printf("realloc %x:%x\n", (int)ptr, size);
	newptr = malloc(size);
	memcpy(newptr,ptr,size);
	free(ptr);
	return(newptr);
}


void free(void *ptr)
{
	_(OBJ_TASK, _mem_free, Ptr(ptr), ___, ___);
}

void exit(int status)
{
	_exit(status);
}

void abort(void)
{
	exit(-1);
}

int setjmp(jmp_buf env)
{
	printf("setjmp\n");
	return(0);
}

void longjmp(jmp_buf env, int val)
{
	printf("longjmp\n");
}

char *getenv(const char *name)
{
	printf("getenv(%s)\n", name);
	return(NULL);
}
