/*  name.c                                                      (CRC:c2e3edcd)
 *      relative path: ./sys
 *      creation date: 1992
 *      last modified: Wed Dec 18 07:18:12 1996
 *      header update: Wed Dec 18 07:18:12 1996
 *  
 *       Microkernel system files
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* name.c
 *   name services
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <error.h>
#include <string.h>
#include <stdio.h>
#include <sys/label.h>


typedef struct name_tree {
	struct name_tree *left;
	struct name_tree *right;
	obj_t value;
	obj_t class;
	char str[4];
} NameTree;


static NameTree *name_root;
static obj_t nextlabel;

obj_t Label_svc(char *str);
obj_t ObjString_svc(char *buf, int len, obj_t id);
obj_t LabelRegister(char *str, obj_t value, obj_t class);
NameTree *FindLabel(NameTree *root, obj_t id);


label_t _label, _method, _object, _error;


obj_t Label_svc(char *str)
{
NameTree *nt;
int n;
char *cp;
	cp = str;
	n = 0;
	while(*cp) {
		if(*cp < '0' || *cp > '9')
			break;
		n = n*10 + (*cp-'0');
		cp++;
	}
	if(!*cp)
		return(n);
	nt = name_root;
	while(nt) {
		n = strcmp(str,nt->str);
		if(!n)
			break;
		if(n<0)
			nt = nt->left;
		else
			nt = nt->right;
	}
	if(nt)
		return(nt->value);
	else
		return(LabelRegister(str,nextlabel++,_label));	
}


obj_t ObjString_svc(char *buf, int len, obj_t id)
{
NameTree *nt;
	nt = FindLabel(name_root,id);
	if(nt) {
		strncpy(buf,nt->str,len);
		return(OK);
	} 
	switch(objtype(id)) {
	case TYPE_INT:
		sprintf(buf,"%d",objdata(id));
		return(OK);
	case TYPE_NINT:
		sprintf(buf,"%d",-objdata(id));
		return(OK);
	}
	sprintf(buf,"<%8x>",id);
	return(ERR_NOATTR);
}


NameTree *FindLabel(NameTree *root, obj_t id)
{
NameTree *s;
	if(!root)
		return(NULL);
	if(root->value == id)
		return(root);
	if((s=FindLabel(root->left,id)))
		return(s);
	return(FindLabel(root->right,id));
}


obj_t LabelRegister(char *str, obj_t value, obj_t class)
{
NameTree **nt;
int n;
	nt = &name_root;
	while(*nt) {
		n = strcmp(str,(*nt)->str);
		if(!n)
			break;
		if(n<0)
			nt = &(*nt)->left;
		else
			nt = &(*nt)->right;
	}
	if(!*nt) {
		*nt = (NameTree*)ksmalloc(sizeof(NameTree)+strlen(str)-3);
		(*nt)->left = NULL;
		(*nt)->right = NULL;
		(*nt)->class = class;
		strcpy((*nt)->str, str);
	}
	(*nt)->value = value;
	return(value);
}


obj_t LabelClassify(obj_t id, obj_t class)
{
NameTree *nt;
	nt = FindLabel(name_root,id);
	if(nt) {
		nt->class = id;
		return(OK);
	} 
	return(ERR_NOATTR);
}


static struct init_label obj_init[] = {
	{"OBJ_NULL", OBJ_NULL},
	{"OBJ_TRUE", OBJ_TRUE},
	{"OBJ_FALSE", OBJ_FALSE},
	{"OBJ_ROOT",OBJ_ROOT},
	{"OBJ_CWD",OBJ_CWD},
	{"OBJ_SYS", OBJ_SYS},
	{"OBJ_THREAD", OBJ_THREAD},
	{"OBJ_TASK", OBJ_TASK},
	{"OBJ_STDIN", OBJ_STDIN},
	{"OBJ_STDOUT", OBJ_STDOUT},
	{"OBJ_STDERR", OBJ_STDERR}};

static struct init_label err_init[] = {
	{"OK", OK},
	{"ERR_FAILURE", ERR_FAILURE},
	{"ERR_INVPARAM", ERR_INVPARAM},
	{"ERR_INVTYPE", ERR_INVTYPE},
	{"ERR_SHORTFILE", ERR_SHORTFILE},
	{"ERR_EOF", ERR_EOF},
	{"ERR_EOB", ERR_EOB},
	{"ERR_NOMEM", ERR_NOMEM},
	{"ERR_NOBUF", ERR_NOBUF},
	{"ERR_NOMETHOD", ERR_NOMETHOD},
	{"ERR_NOATTR", ERR_NOATTR},
	{"ERR_EXISTATTR", ERR_EXISTATTR},
	{"ERR_EOA", ERR_EOA},
	{"ERR_INVOBJ", ERR_INVOBJ},
	{"ERR_NOREAD", ERR_NOREAD},
	{"ERR_NOWRITE", ERR_NOWRITE},
	{"ERR_NOTHREAD", ERR_NOTHREAD},
	{"ERR_NOTASK", ERR_NOTASK}};


extern struct init_label label_init[];
extern int label_init_size;

void Name_init(void)
{
int n;
struct init_label *np;
	name_root = NULL;
	nextlabel = OBJ_LABEL_MIN;
	_label = Label("label");
	LabelClassify(_label, _label);
	_method = Label("method");
	_object = Label("object");
	_error = Label("error");
	np = obj_init;
	n = arraysize(obj_init);
	while(n--) {
		LabelRegister(np->str, np->val, _object);
		np++;
	}
	np = err_init;
	n = arraysize(err_init);
	while(n--) {
		LabelRegister(np->str, np->val, _error);
		np++;
	}
	np = label_init;
	n = label_init_size;
	while(n--) {
		*(label_t*)(np->val) = Label(np->str);
		np++;
	}
}


