/*  object.c                                                    (CRC:c7971c91)
 *      relative path: ./sys
 *      creation date: 1992
 *      last modified: Wed Dec 18 07:18:13 1996
 *      header update: Wed Dec 18 07:18:13 1996
 *  
 *       Microkernel system files
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* object.c
 *  object-oriented kernel extensions
 */


#define KERNEL
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/global.h>
#include <object.h>
#include <error.h>

extern label_t _reboot, _halt;

void ObjCall_err(vaddr pc, obj_t rv, obj_t oid, obj_t op, obj_t arg1, obj_t arg2, obj_t arg3);


static obj_t sysobj;


static obj_t Object_eval(obj_t oid)
{
register int i;
	if(objtype(oid) != TYPE_OBJ)
		return(oid);
	switch(oid) {
	case OBJ_ROOT:		return(thread->root);
	case OBJ_CWD:		return(thread->cwd);
	case OBJ_SYS:		return(sysobj);
	case OBJ_KERNEL:	return(kernel.self);
	case OBJ_THREAD:	return(thread->self);
	case OBJ_TASK:		
		if(thread->task)
			return(thread->task->self);
		else
			return(OBJ_NULL);
	}
	i = objdata(oid);
	if(i >= OBJ_THREAD_MIN && i <= OBJ_THREAD_MAX)
		return(thread->std_obj[i-OBJ_THREAD_MIN]);
	return(oid);
}

static Object *Object_ptr(obj_t oid)
{
register int i;
	oid = Object_eval(oid);
	if(objtype(oid) != TYPE_OBJ)
		return(NULL);
	i = objdata(oid);
	if(thread && 0 /* XXX */) {
		if(i < thread->task->objtable_size)
			return(&thread->task->objtable[i]);
	} else {
		if(i < kernel.objtable_size)
			return(&kernel.objtable[i]);
	}
	return(NULL);
}


obj_t ObjCall_svc(register struct objcall_frame *callframe,
			 obj_t errchk, obj_t self, vaddr pc)
{
obj_t retval;
Object *optr;
ObjFrame frame;
ObjClass *class;
MethodTable *method; 
obj_t oid;
int n;
	retval = ERR_NOMETHOD;
	frame.self = Object_eval(self);
	oid = callframe->oid;
	while(oid != OBJ_NULL) {
		if(!(optr = Object_ptr(oid))) {
			retval = ERR_INVOBJ;
			break;
		}
		class = optr->class;
		
		/* XXX - security hole-- needs to enforce user privs
		 * while looking up or setting attributes */
		if(callframe->op == _getattr)
			if(callframe->arg1 < class->nattr) 
				return(*(class->attr[callframe->arg1].id));
			else
				callframe->arg1 -= class->nattr;
		method = class->method;
		/* XXX - possible sync problems, if two threads make
		 a call using the same object-- one could modify a value,
		 and the buffering for the other could restore it... */
		retval = ERR_NOMETHOD;
		n = class->nmethod;
		while(n--) {
			if(*(method->id) == callframe->op || *(method->id) == OBJ_ANY) {
				frame.next = optr->next;
				frame.data = optr->data;
				frame.size = optr->size;
				frame.oid = oid;
				frame.op = callframe->op;
				if(!(optr->flags & OFLAG_KERNEL)) {
					kprintf("to user...(%x,%x,%x)\n",
					oid,callframe->op,callframe->arg1);
					retval = call_user(
						(vaddr)method->func, &frame,
						frame.data, callframe->arg1,
						callframe->arg2,
						callframe->arg3);
					kprintf("back\n");
				} else 
					retval = method->func(&frame,
						frame.data,callframe->arg1,
						callframe->arg2,
						callframe->arg3);
				/* XXX - this is slow, but if we don't
				 * get optr again, it will be wrong when
				 * the table gets relocated */
				optr = Object_ptr(oid);
				optr->data = frame.data;
				optr->size = frame.size;
				break;
			}
			method++;
		}
		if(retval != ERR_NOMETHOD && retval != ERR_NOATTR)
			break;
		/* if(op == _GETATTR && retval == ERR_EOA)
			subtract number of attributes and then
			call next object with that value
		else */ if(retval != ERR_NOMETHOD && retval != ERR_NOATTR)
			break;
		if(callframe->op == _get)
			retval = AttrGet(optr->data,class->attr,class->nattr,
					callframe->arg1);
		else if(callframe->op  == _set)
			retval = AttrSet(optr->data,class->attr,class->nattr,
					callframe->arg1,callframe->arg2);
		if(retval != ERR_NOMETHOD && retval != ERR_NOATTR)
			break;
		oid = optr->next;
	} 
	if(errchk && err(retval))
		ObjCall_err(pc,retval,callframe->oid,
			callframe->op,callframe->arg1,
			callframe->arg2,callframe->arg3);
	return(retval);
}


void ObjCall_err(vaddr pc, obj_t rv, obj_t oid, obj_t op, obj_t arg1, obj_t arg2, obj_t arg3)
{
char buf1[20],buf2[20],buf3[20],buf4[20];
	ObjString(buf1,sizeof(buf1),op);
	ObjString(buf2,sizeof(buf2),arg1);
	ObjString(buf3,sizeof(buf3),arg2);
	ObjString(buf4,sizeof(buf4),arg3);
	panic(2,
		"ObjCall(%x(%x),%s,%s,%s,%s): %s",
		oid,Object_eval(oid),buf1,buf2,buf3,buf4,
		strerror(rv));
}


static obj_t ObjCreate(ObjClass *class, obj_t self, obj_t next, 
			obj_t arg1, obj_t arg2, obj_t arg3)
{
Object *optr;
obj_t oid;
obj_t rv;
Task *task;
int i,n;
	if(thread && 0)
		task = thread->task;
	else
		task = &kernel;
	LOCK(task->objlock);
	if(!task->objtable_size) {
		task->objtable_free = -1;
		task->objtable = (Object*)vm_alloc(&task->data,
					PAGE_SIZE,VM_KRW);
	}
	if(task->objtable_free < 0) {
		i = task->objtable_size;
		n = i + PAGE_SIZE/sizeof(Object);
		task->objtable = (Object*)vm_realloc(&task->data,
				(vaddr)task->objtable,n*sizeof(Object));
		task->objtable_size = n;
		task->objtable_free = i;
		while(i < n-1) {
			task->objtable[i].next = (obj_t)(i+1);
			i++;
		}
		task->objtable[i].next = (obj_t)(-1);
	}
	n = task->objtable_free;
	optr = &task->objtable[n];
	task->objtable_free = (int)(optr->next);
	oid = Obj(TYPE_OBJ, n);

	if(is_kernel())
		optr->flags = OFLAG_KERNEL;
	else	
		optr->flags = 0;
	optr->ref = 1;
	optr->class = class;
	optr->next = next;
	optr->data = 0;
	optr->size = 0;
	if(self == OBJ_NULL)
		self = oid;
	rv = ObjCall_Self(oid,_init,arg1,arg2,arg3,self);
	if(err(rv)) {
		kprintf("obj_init failed: %x\n", rv);
		/* XXX - should "uncreate" object */
		UNLOCK(task->objlock);
		return(rv);
	}
	if(!(is_kernel())) {
		kprintf("not kernel?!?n");
		rv &= ~OFLAG_KERNEL;
	}
	optr->flags = rv;
	UNLOCK(task->objlock);
	return(oid);
}


ulong AttrGet(void *data, AttrTable *table, int tsize, obj_t id)
{
	while(tsize && (*(table->id) != id 
			|| (table->type&ATYPE_NOREAD)) ) {
		table++;
		tsize--;
	}
	if(!tsize)
		return(ERR_NOATTR);
	switch(table->type&ATYPE_TYPE) {
	case ATYPE_CHAR:
		return(*((char*)((vaddr)data+table->value)));
	case ATYPE_UCHAR:
		return(*((uchar*)((vaddr)data+table->value)));
	case ATYPE_SHORT:
		return(*((short*)((vaddr)data+table->value)));
	case ATYPE_USHORT:
		return(*((ushort*)((vaddr)data+table->value)));
	case ATYPE_LONG:
	case ATYPE_ULONG:
	case ATYPE_OBJ:
		return(*((ulong*)((vaddr)data+table->value)));
	default:
		return((*((ulong*)((vaddr)data+table->value))
			 >> (table->type&31)) & 1);
	}
}


ulong AttrSet(void *data, AttrTable *table, int tsize, obj_t id, obj_t val)
{
ulong old;
int bit;
	while(tsize && (*(table->id) != id 
			|| (table->type&ATYPE_NOWRITE)) ) {
		table++;
		tsize--;
	}
	if(!tsize)
		return(ERR_NOATTR);
	switch(table->type&ATYPE_TYPE) {
	case ATYPE_CHAR:
		old = *((char*)((vaddr)data+table->value));
		*((char*)((vaddr)data+table->value)) = val;
		break;
	case ATYPE_UCHAR:
		old = *((uchar*)((vaddr)data+table->value));
		*((uchar*)((vaddr)data+table->value)) = val;
		break;
	case ATYPE_SHORT:
		old = *((short*)((vaddr)data+table->value));
		*((short*)((vaddr)data+table->value)) = val;
		break;
	case ATYPE_USHORT:
		old = *((ushort*)((vaddr)data+table->value));
		*((ushort*)((vaddr)data+table->value)) = val;
		break;
	case ATYPE_LONG:
	case ATYPE_ULONG:
	case ATYPE_OBJ:
		old = *((ulong*)((vaddr)data+table->value));
		*((ulong*)((vaddr)data+table->value)) = val;
		break;
	default:
		bit = (table->type)&31;
		old = (*((ulong*)((vaddr)data+table->value))
			 >> bit) & 1;
		if(val && val != OBJ_FALSE && val != OBJ_NULL)
			*((ulong*)((vaddr)data+table->value)) |=
				(1 << bit);
		else
			*((ulong*)((vaddr)data+table->value)) &=
				~(1 << bit);
	}
	return(old);
}



static obj_t object_init(ObjFrame *frame)
{
	return(OFLAG_KERNEL);
}


static obj_t object_new(ObjFrame *frame, obj_t data, obj_t arg1,
					obj_t arg2, obj_t arg3)
{
Object *optr;
obj_t new;
	optr = Object_ptr(frame->self);
	new = ObjCreate(optr->class, frame->self, optr->next, arg1, arg2, arg3);
	return(new);
}


static obj_t object_next(ObjFrame *frame, obj_t data, obj_t obj)
{
Object *optr,*nptr;
	if(!(optr = Object_ptr(frame->self)))
		return(ERR_NOMETHOD);
	if(!(nptr = Object_ptr(obj)))
		return(ERR_NOMETHOD);
	nptr->next = optr->next;
	optr->next = obj;
	return(OK);
}

 
static obj_t object_free(ObjFrame *frame)
{
obj_t oid;
Object *optr;
int n, i;
Task *task;
	if(thread && 0)
		task = thread->task;
	else
		task = &kernel;
	LOCK(task->objlock);
	oid = Object_eval(frame->self);
	if(objtype(oid) != TYPE_OBJ)
		panic(0,"Freeing a non-object");
	i = objdata(oid);
	if(i >= kernel.objtable_size)
		panic(0,"Freeing invalid object");	
	optr = &kernel.objtable[i];
	n = --(optr->ref);
	if(n <= 0) {
		if(optr->flags & OFLAG_HASVEC) 
			kfree((void*)(optr->data));
		optr->next = (int)(task->objtable_free);
		task->objtable_free = i;
	}
	UNLOCK(task->objlock);
	return(n);
}


static obj_t object_eval(ObjFrame *frame)
{
	return(frame->self);
}


static MethodTable object_method[] = {
	{&_init,(obj_method)object_init},
	{&_new,(obj_method)object_new},
	{&_next,(obj_method)object_next},
	{&_free,(obj_method)object_free},
	{&_eval,(obj_method)object_eval}};

static ObjClass Class_Object = {
	object_method,
	NULLP(AttrTable),
	arraysize(object_method),
	0};




static ObjClass Class_Class;


static obj_t class_init(ObjFrame *frame, obj_t data, ObjClass *class)
{
	frame->data = (void*)class;
	return(OFLAG_KERNEL);
}


static obj_t class_new(ObjFrame *frame, obj_t data, obj_t arg1, obj_t arg2, obj_t arg3)
{
obj_t new;
	new = ObjCreate(frame->data,frame->self,frame->next,arg1,arg2,arg3);
	return(new);
}


static obj_t class_newclass(ObjFrame *frame, obj_t data, obj_t arg1, obj_t arg2, obj_t arg3)
{
obj_t new;
	new = ObjCreate(&Class_Class,frame->self,frame->next,arg1,arg2,arg3);
	return(new);
}


static MethodTable class_method[] = {
	{&_init,(obj_method)class_init},
	{&_new,(obj_method)class_new},
	{&_class,(obj_method)class_newclass}};

static ObjClass Class_Class = {
	class_method,
	NULLP(AttrTable),
	arraysize(class_method),
	0};



static obj_t sys_init(ObjFrame *frame)
{
	return(OFLAG_KERNEL);
}


static obj_t sys_write(ObjFrame *frame, obj_t data, char *buf, int len)
{
	return(console_write(buf,len));
}


extern label_t _zone_status, _mem_status, _vm_status, _vm_list;
extern label_t _mem_list, _nuke_fs, _sys_stat, _load_average;

static MethodTable sys_method[] = {
	{&_init,(obj_method)sys_init},
	{&_write,(obj_method)sys_write},
	{&_halt,(obj_method)halt},
	{&_reboot,(obj_method)reboot},
	{&_vm_status,(obj_method)vm_status},
	{&_vm_list,(obj_method)vm_list},
	{&_zone_status,(obj_method)zone_status},
	{&_mem_status,(obj_method)mem_status},
	{&_mem_list,(obj_method)mem_list},
	{&_nuke_fs,(obj_method)nuke_fs},
	{&_sys_stat,(obj_method)sys_stat},
	{&_load_average,(obj_method)load_average}};


static ObjClass Class_Sys = {
	sys_method,
	NULLP(AttrTable),
	arraysize(sys_method),
	0};

void obj_print(obj_t oid)
{
Object *optr;
	if(oid == OBJ_NULL)
		kprintf("<null>");
	while(oid != OBJ_NULL) {
		optr = Object_ptr(oid);
		if(optr)
			kprintf("%x:(c:%x,d:%x,s:%x) ",
				oid, optr->class, optr->data, optr->size);
		else {
			kprintf("<%x> ", oid);
			break;
		}
		oid = optr->next;
	}
	kprintf("\n");
}


void Object_init(void)
{
obj_t object;
	kprintf("obj_init(): ");
	kernel.objtable_size = 0;
	LOCK_CLEAR(kernel.objlock);
	object = ObjCreate(&Class_Object,OBJ_NULL,OBJ_NULL,___,___,___);
	class_class = ObjCreate(&Class_Class,OBJ_NULL,OBJ_NULL,___,___,___);
	class_object = ObjCreate(&Class_Class,OBJ_NULL,object,___,___,___);
	sysobj = ObjCreate(&Class_Sys,OBJ_NULL,OBJ_NULL,___,___,___);
	kprintf("done\r\n");
}

