/*  argparse.c                                                  (CRC:9e44f387)
 *      relative path: ./usr
 *      creation date: 1992
 *      last modified: Wed May 22 21:39:43 1996
 *      header update: Wed Dec 18 07:18:27 1996
 *  
 *       User-level programs
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* argparse.c 
 *   argument parsing routine to break down a command line
 *   into individual arguments 
 *   (currently does not handle wildcard matching)
 *   Project JakeDev for Computer Science House
 *   December 11th, 1990
 *   by Frank Barrus 
 */


#define  is_num(x)    ((x >= '0' && x <= '9') || x == '-')
#define  is_word(x)   ((x >= 'A' && x <= 'Z') || (x >= 'a' && x <= 'z') \
			|| (x == '_'))
#define  is_punc(x)   ((x) && strchr("!@#$%^*?+-/\\.:\"",x))
#define  is_sep(x)    ((x) && strchr("&()[]<>{}|x`',=;",x))
#define  is_sym(x)    (is_num(x) || is_word(x) || is_punc(x))


static int argnext(char *wbuf, char *str, int pos)
{
boolean inquote = FALSE;
int wpos = 0;
   while(str[pos] == ' ')
      pos++;
   while(inquote || is_sym(str[pos])) {
      if(str[pos]=='"') {
         inquote = !inquote;
         pos++;
	 continue;
      }
      if(str[pos]=='\\') 
         pos++;
      wbuf[wpos++] = str[pos++];
   } 
/* might be problem if str[pos] = 0 */
   if(is_sep(str[pos]) && (!wpos))
      wbuf[wpos++] = str[pos++];
   wbuf[wpos] = '\0';
   return(pos);
}

static int argparse(char *str, char **argv, char *argbuf, int trans)
{
int p=0;
int n=0;
int i=0;
   do {
      argv[n] = &argbuf[i];
      p=argnext(argv[n],str,p);
      i += strlen(argv[n])+1;
   } while(*argv[n++]);
   return(n-1);
}

