/*  tip.c                                                       (CRC:d64a2ec3)
 *      relative path: ./usr
 *      creation date: 1992
 *      last modified: Wed May 22 21:40:01 1996
 *      header update: Wed Dec 18 07:18:35 1996
 *  
 *       User-level programs
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* tip.c
 *  simple terminal program
 */

#include <sys/types.h>
#include <object.h>
#include <error.h>
#include <stddef.h>
#include <stdio.h>
#include <dir.h>
#include <event.h>


extern label_t _speed, _echo, _lflag, _rnoblock, _break, _icanon, _isig;
extern label_t _stdin, _msleep, _hangup;
extern label_t _suspend, _signal, _parent;

static void tip_connect(obj_t line)
{
char ch, lastch;
char buf[512]; 
int len;
ulong lflag;
boolean tilde, quit;
boolean active;
obj_t rblock, fdin;
obj_t pid, ppid;
	tilde = FALSE;
	quit = FALSE;
	printf("Entering terminal mode- type ~? for help...\n");
	lflag = _(OBJ_STDIN,_get,_lflag, ___, ___);
	_(OBJ_STDIN,_set,_icanon,OBJ_FALSE, ___);
	_(OBJ_STDIN,_set,_isig,OBJ_FALSE, ___);
	_(OBJ_STDIN,_set,_echo,OBJ_FALSE, ___); 
	_(line,_set,_icanon,OBJ_FALSE, ___);
	_(line,_set,_isig,OBJ_FALSE, ___);
	_(line,_set,_echo,OBJ_FALSE, ___);
	fdin = _(line,_get,_stdin,___,___);
	_(fdin,_set,_rnoblock,OBJ_TRUE, ___);
	fdin = _(OBJ_STDIN,_get,_stdin,___,___);
	rblock = _(fdin,_get,_rnoblock,___,___);
	_(fdin,_set,_rnoblock,OBJ_TRUE, ___);
	lastch = '\0';
	while(!quit) {
		active = FALSE;
		if ((len=_(line,_read,Ptr(buf),sizeof(buf), ___)) > 0) {
			active = TRUE;
			_(OBJ_STDOUT, _write, Ptr(buf), len, ___);
		}
		while((len=_(OBJ_STDIN, _read, Ptr(&ch),1, ___)) > 0) {
			active = TRUE;
			if (ch == '~' && lastch == '\r')
				if ((tilde = !tilde))
					continue;
			if (tilde) {
				switch(ch) {
				case '.': 
					quit = TRUE; break;
				case '#': 
					err_(line,_break, ___, ___, ___); 
					break;
				case '@':
					err_(line,_hangup, ___, ___, ___);
					break;
				case 'Z'-64:
					pid = _(OBJ_THREAD, _eval, ___, ___, ___);
					ppid = _(OBJ_THREAD, _get, _parent, ___, ___);
					_(ppid, _signal, EVENT_CHILDSTOP, pid, pid);
					_(OBJ_THREAD, _suspend, ___, ___, ___);
					break;
				default:
					printf("commands:\r\n");
					printf(" ~?	help\r\n");
					printf(" ~#	break\r\n");
					printf(" ~@	hangup\r\n");
					printf(" ~^z	suspend\r\n");
					printf(" ~.	exit\r\n");
				}
				tilde = FALSE;
			} else {
				lastch = ch;
				_(line, _write, Ptr(&ch), len, ___);
			}
		}
		if(!active)
			_(OBJ_THREAD, _msleep, 0, ___, ___);
	}
	_(OBJ_STDIN,_set,_lflag,lflag, ___);
	err_(fdin,_set,_rnoblock,rblock, ___);
}


obj_t Main(ObjFrame *frame, vaddr base, obj_t arg, obj_t env)
{
obj_t wd,fd;
/*
	if(argc < 2) {
		printf("usage: tip <port> [<speed>]\n");
		return(ERR_NOMETHOD);
	}
*/
	wd = dir_parse(OBJ_ROOT,"/dev");
	fd = dir_parse(wd, "/dev/ttys1");
	if(err(fd))
		return(fd);
/*	if(argc > 2 || 1)  
		err_(fd,_set,_speed,38400, ___); */
	tip_connect(fd);
	return(OK);
}

