/*  cutsym.c                                                    (CRC:fbd2004d)
 *      relative path: ./util
 *      creation date: 1992
 *      last modified: Wed May 22 21:40:14 1996
 *      header update: Wed Dec 18 07:18:39 1996
 *  
 *       Utilities
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* cutsym.c
 *   cuts out just the header and symbol table from a file
 *   takes input from standard in, and writes to standard out
 *   is currently limited to a 64K maximum size for the symbol table
 */

#include <sys/types.h>
#include <sys/exec.h>

#define SYM_MAX 65536

main()
{
struct exec header;
char *buf;
int n;
ulong strsize;
	buf = (char*)malloc(SYM_MAX);
	read(0,&header,sizeof(header));
	if(header.a_magic == ZMAGIC || header.a_magic == NMAGIC)
		read(0,buf,1024-sizeof(header));
	n = header.a_text+header.a_data+header.a_trsize+header.a_drsize;
	if(header.a_magic == QMAGIC)
		n -= sizeof(header);
	while(n>0) {
		read(0,buf,((n>SYM_MAX)?SYM_MAX:n));
		n -= SYM_MAX;
	}
	read(0,buf,header.a_syms);
	read(0,&strsize,4);
	n = read(0,buf+header.a_syms,strsize-4);
	if(n < strsize-4) {
		write(2,"unexpected EOF\n",15);
		exit(1);
	}
	header.a_magic = OMAGIC;
	header.a_text = 0;
	header.a_data = 0;
	header.a_bss = 0;
	header.a_trsize = 0;
	header.a_drsize = 0;
	write(1,&header,sizeof(header));
	write(1,buf,header.a_syms);
	write(1,&strsize,4);
	write(1,buf+header.a_syms,strsize-4);
	return(0);
}

