/*  setshared.c                                                 (CRC:9fa96b12)
 *      relative path: ./util
 *      creation date: 1992
 *      last modified: Wed May 22 21:41:02 1996
 *      header update: Wed Dec 18 07:18:47 1996
 *  
 *       Utilities
 *  
 *  Copyright (c) 1992 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is a part of the Shag/OS operating system distribution, and is
 *  free software for non-profit uses; it may be freely used and distributed
 *  for non-profit purposes under the terms of the Shag/OS License, version 1.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND, 
 *  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 *  AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT WILL THE AUTHOR, OR
 *  ANYONE RESPONSIBLE FOR THE LEGAL MODIFICATION OR REDISTRIBUTION OF THIS
 *  SOFTWARE, BE LIABLE FOR ANY DAMAGES, INCLUDING BUT NOT LIMITED TO ANY LOST
 *  PROFITS, LOST SAVINGS OR ANY INCIDENTAL OR CONSEQUENTIAL DAMAGES, WHETHER
 *  RESULTING FROM IMPAIRED OR LOST DATA, SOFTWARE OR COMPUTER FAILURE OR ANY
 *  OTHER CAUSE, EVEN IF THE AUTHOR IS ADVISED OF THE POSSIBILITY OF SUCH
 *  DAMAGES, OR FOR ANY OTHER CLAIM BY THE USER OR FOR ANY THIRD PARTY CLAIM.
 *  
 *  You should have received a copy of the Shag/OS License along with this
 *  program, in a file called 'license1.txt'.  If this license is missing,
 *  please contact the author and describe how this software was obtained.
 *  
 *  The author can be reached for questions, comments, and suggestions at:
 *  
 *      Frank Barrus, 4435 River Rd. Apt #3, Scottsville, NY 14546, USA
 *      email:   shaggy@csh.rit.edu
 *  
 *      For sending bug reports, please read the file 'bugs', and always
 *      include the product name and version, and the word 'BUG' in the subject.
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/csh/shaggy
 */

/* setshared.c
 *  utility to selectively mark symbol table entries as "shared"
 */

#include <sys/types.h>
#include <sys/exec.h>
#include <stdio.h>

#define MAXSHARED	4096	/* maximum number of shared entries to mark */


main(int argc, char **argv)
{
FILE *f;
char **shared;
char str[200];
int nshared;
char *strx;
int strsize;
struct exec header;
struct nlist *sym;
struct nlist *nl;
int i,n,nfound;
char *s;
int hsize;
	shared = (char**)malloc(MAXSHARED*sizeof(char*));
	nshared = 0;
	f = fopen(argv[2],"r");
	while(fgets(str,sizeof(str),f)) {
		str[strlen(str)-1] = '\0';
		if(!str[0] || str[0] == '#')
			continue;
		if(str[0] == '.') {
			shared[nshared] = (char*)malloc(strlen(str));
			strcpy(shared[nshared],str+1);
		} else {
			shared[nshared] = (char*)malloc(strlen(str)+2);
			strcpy(shared[nshared]+1, str);
			shared[nshared][0] = '_';
		}
		nshared++;
	}
	fclose(f);
	f = fopen(argv[1],"r+");
	fread(&header,sizeof(header),1,f);
	if(header.a_magic == NMAGIC || header.a_magic == ZMAGIC)
		hsize = 1024;
	else if(header.a_magic == QMAGIC)
		hsize = 0;
	else
		hsize = sizeof(header);
	fseek(f,hsize+header.a_text+header.a_data+header.a_trsize+header.a_drsize,
			SEEK_SET);
	sym = (struct nlist*)malloc(header.a_syms);
	fread(sym,header.a_syms,1,f);
	fread(&strsize,sizeof(int),1,f);
	strx = (char*)malloc(strsize);
	fread(strx+sizeof(int),strsize-sizeof(int),1,f);
	nfound = 0;	
	for(n=0,nl=sym; n<header.a_syms/sizeof(struct nlist); n++,nl++) {
		s = strx+nl->n_strx;
		for(i=0; i<nshared; i++)
			if(!strcmp(s,shared[i])) {
				nfound++;
				nl->n_type |= N_SHARE;
				shared[i][0] = '\0';
				break;
			}
	}

	if(nfound < nshared) {
		printf("Not found:\n");	
		for(i=0; i<nshared; i++)
			if(shared[i][0]) 
				printf("    %s\n", shared[i]);
	}

	fseek(f,hsize+header.a_text+header.a_data+header.a_trsize+header.a_drsize,
			SEEK_SET);
	fwrite(sym,header.a_syms,1,f);
	fclose(f);
}

